# Blazor Internationalization(I18n) Text [![NuGet Package](https://img.shields.io/nuget/v/Toolbelt.Blazor.I18nText.svg)](https://www.nuget.org/packages/Toolbelt.Blazor.I18nText/) [![Discord](https://img.shields.io/discord/798312431893348414?style=flat&logo=discord&logoColor=white&label=Blazor%20Community&labelColor=5865f2&color=gray)](https://discord.com/channels/798312431893348414/1202165955900473375)

## 📝 Summary

This package is another way to localize text in your Blazor Web App!

![movie.1](https://raw.githubusercontent.com/jsakamoto/assets/m/i18n/movie1.gif)

- [Live Demo is here!](https://demo-blazor-i18ntext.azurewebsites.net/)

### 🎁 Features

- All of the render modes (SSR, Server, WebAssembly, and Auto) are supported.
- In Blazor Wasm, it works even on a static web host. (The ASP.NET Core host isn't required)
- Only needs a plain text editor - no need for .resx files
- Static Typing - IntelliSense, Code Hint, etc.
- It supports Blazor components libraries. You can create NuGet packages of your libraries that are localized with "Blazor I18nText".

### 📢 Notice

Blazor WebAssembly has started localization support officially since v.3.2 preview 4. It is based on .NET Standard `IStringLocalizer` and satellite assemblies with `.resx`.

However, I will continue to develop and maintain this package, because this package still has some advantages over the .NET standard implementation.

![pros/cons](https://raw.githubusercontent.com/jsakamoto/assets/m/i18n/pros-cons.png)

### Supported .NET versions

.NET 6.0, 8.0, or later versions.

## 🚀 Quick Start

### Step 1 - Add Package

Add `Toolbelt.Blazor.I18nText` NuGet package to your Blazor app project as shown below

```shell
dotnet add package Toolbelt.Blazor.I18nText
```

### Step 2 - Create localized text source files as JSON or CSV

Add localized text source files for each language in an `i18ntext` folder under the project folder.

The localized text source files must be simple key-value only JSON files, as shown in the example below:

```json
{
  "Key1": "Localized text 1",
  "Key2": "Localized text 2",
  ...
}
```

or, an only 2-column CSV file without a header row as shown in the example below.

```
Key1,Localized text 1
Key2,Localized text 2
```

> [!IMPORTANT]  
> The encoding of the CSV and JSON files must be **UTF-8**.

The naming rule for the localized text source files must follow this format:

```text
<Text Table Name>.<Language Code>.{json|csv}
```

For example, the JSON-formatted localized text source file for the English language of the `MyText` Text Table class must be named `MyText.en.json`.

![fig.1](https://raw.githubusercontent.com/jsakamoto/assets/m/i18n/fig1.png)

### Step 3 - Build the project whenever localized text source files are created or updated.

Basically, after creating or updating those localized text source files, **you may have to build your Blazor app project.**

After that, **"Typed Text Table class" C# source code** will be generated by the C# "source generator" feature.

And also, **"Localized Text Resource JSON" files** will be generated in the output folder.

![fig.2](https://raw.githubusercontent.com/jsakamoto/Toolbelt.Blazor.I18nText/master/.assets/fig.002b.png)

#### for Visual Studio IDE users

On Visual Studio IDE, the source generator will work automatically in the background. Therefore, **Visual Studio IDE users usually do not need to build the project explicitly.**

Localized text source files (.json or .csv) are recompiled into "Typed Text Table class" and "Localized Text Resource JSON" files automatically whenever they are updated.

#### for dotnet CLI users

When you are developing on dotnet CLI and want to recompile localized text source files automatically whenever they are changed, you can use the `dotnet watch` command.

```shell
$ dotnet watch
```

After entering that dotnet CLI command, the command window will stay in execution mode and watch for changes in the localized text source files.

When that dotnet CLI detects localized text source files changing, the dotnet CLI will recompile localized text source files into **"Typed Text Table class"** and **"Localized Text Resource JSON"** files.

![fig.2-2](https://raw.githubusercontent.com/jsakamoto/Toolbelt.Blazor.I18nText/master/.assets/fig.002-2b.png)

### Step 4 - Configure your app to use I18nText service

Edit the "Program.cs" file to register "I18nText" service, like this.

```csharp
// in your Program.cs
using Toolbelt.Blazor.Extensions.DependencyInjection; // 👈 Add this, and...
...
var builder = WebApplication.CreateDefault(args);
...

// 👇 Add the following code to register the I18nText service.
builder.Services.AddI18nText(
    options => options.PersistenceLevel = PersistanceLevel.Cookie);
...
```

> [!NOTE]
> If your Blazor app has SSR areas, we strongly recommend to use `PersistanceLevel.Cookie` option to keep the language settings, because it is the only way to keep the language settings in the SSR areas. If your Blazor app doesn't have any SSR areas, you can use other options, such as `PersistanceLevel.Session`, etc.

If your Blazor app has SSR areas, or the Server-side Pre-rendering is enabled, or the `I18n Text` options are configured to use cookies to persist language settings, you have to add the following code to configure the `RequestLocalization` middleware, and use the `RequestLocalization` middleware, like this.

```csharp
// in your Program.cs

...
// 👇 Add the following code to configure
//     the RequestLocalization middleware, and...
builder.Services.Configure<RequestLocalizationOptions>(options =>
{
    // This is an example.
    // You should configure the supported cultures as you like.
    var supportedCultures = new[] { "en", "ja" };
    options.DefaultRequestCulture = new("en");
    options.AddSupportedCultures(supportedCultures);
    options.AddSupportedUICultures(supportedCultures);
});

...
var app = builder.Build();
...

// 👇 Add the following code to use the RequestLocalization middleware.
app.UseRequestLocalization();
...
```

### Step 5 - Get the "Text Table" object in your Blazor component

Open your Blazor component file (.razor) inside any editor, and do the following:

1. Inject `Toolbelt.Blazor.I18nText.I18nText` service into the component.

```csharp
@inject Toolbelt.Blazor.I18nText.I18nText I18nText
```

2. Add a field of the Text Table class generated from localized text source files, and assign the default instance.

```csharp
@code {
  private I18nText.MyText MyText = new();
  ...
```

> [!NOTE]
> The namespace of the Text Table class is `<default namespace of your Blazor project>` + `"I18nText"`.

3. Override `OnInitiallizedAsync()` method of the Blazor component, and assign a Text Table object that's a return value of `GetTextTableAsync<T>()` method of `I18nText` service instance to the Text Table field.

```csharp
...
protected override async Task OnInitializedAsync()
{
  MyText = await I18nText.GetTextTableAsync<I18nText.MyText>(this);
  ...
```

![fig.4](https://raw.githubusercontent.com/jsakamoto/assets/m/i18n/fig4.png)

### Step 6 - Use the Text Table

After doing these steps, you can reference a field of the Text Table object to get localized text.

If you are using Visual Studio in Windows OS or Visual Studio Code with the C# extension, you will get "IntelliSense" and "Document comment" support.

![movie.2](https://raw.githubusercontent.com/jsakamoto/assets/m/i18n/movie2.gif)

> [!TIP]
> Text Table object allows you to get localized text by key string dynamically, with indexer syntax, like this.
>
> ```html
> <h1>@MyText["HelloWorld"]</h1>
> ```
>
> If you make some mistakes that typo of key string, it will just return the key string as is without any runtime exceptions.

### Step 7 - Run it!

Build and run your Blazor app.

The I18nText service detects the language settings of the Web browser, and reads the localized text resource JSON which is most suitable for the language detected.

![fig.5](https://raw.githubusercontent.com/jsakamoto/assets/m/i18n/fig5.png)

## 📌 Limitations

The following features are not supported in this version of `I18Text` library.

- Integration with ASP.NET Core localization (`IStringLocalizer<T>` support)
- Localize validation message
- Plural form support
- Text formatting by placeholder.
- Integration with `System.Globalization.Culture.CurrentUICulture`.

The following features will not be supported forever, because these features are not the scope of this library, we think.

- Formatting of date, time, currency. (These features will be provided by `System.Globalization.Culture`.)

## ⚙️ Configuration

- [Fallback language](https://github.com/jsakamoto/Toolbelt.Blazor.I18nText/blob/master/README-CONFIGURATION.md#fallback-language)
- [The namespace of the Text Table class](https://github.com/jsakamoto/Toolbelt.Blazor.I18nText/blob/master/README-CONFIGURATION.md#the-namespace-of-the-text-table-class)

## 🛠️ API Reference

Please see also ["API Reference"](https://github.com/jsakamoto/Toolbelt.Blazor.I18nText/blob/master/API-REFERENCE.md#api-reference) on GitHub.

## 🎉 Release Notes

Release notes are [here.](https://github.com/jsakamoto/Toolbelt.Blazor.I18nText/blob/master/RELEASE-NOTES.txt)

## 📢 Licence & Third Party Notice

- [Mozilla Public Licence Version 2.0](https://github.com/jsakamoto/Toolbelt.Blazor.I18nText/blob/master/LICENSE)
- [Third party notice](https://github.com/jsakamoto/Toolbelt.Blazor.I18nText/blob/master/THIRDPARTYNOTICES.txt)
